@@ -13,6 +13,7 @@ from .obstruction_inferral import (
 from .obstruction_transitivity import ObstructionTransitivity
 from .requirement_placement import RequirementPlacement
 from .row_col_separation import RowColSeparation
+from .sliding import Sliding
 from .subclass_verification import SubclassVerificationAlgorithm
 
 __all__ = [
@@ -35,4 +36,5 @@ __all__ = [
     "RowColSeparation",
     "SubclassVerificationAlgorithm",
     "guess_obstructions",
+    "Sliding",
 ]
