@@ -2,7 +2,7 @@ from collections import Counter, defaultdict
 from functools import reduce
 from itertools import chain
 from operator import mul
-from typing import Dict, Iterable, Iterator, List, Optional, Tuple, cast
+from typing import Dict, Iterable, Iterator, List, Optional, Tuple, Type, TypeVar, cast
 
 from sympy import Expr, Function, var
 
@@ -21,6 +21,7 @@ from permuta.permutils import (
 )
 from permuta.permutils.symmetry import all_symmetry_sets
 from tilings import GriddedPerm, Tiling
+from tilings.algorithms import locally_factorable_shift
 from tilings.algorithms.enumeration import (
     DatabaseEnumeration,
     LocalEnumeration,
@@ -36,6 +37,7 @@ from tilings.strategies import (
 x = var("x")
 
 __all__ = [
+    "BasisAwareVerificationStrategy",
     "BasicVerificationStrategy",
     "OneByOneVerificationStrategy",
     "DatabaseVerificationStrategy",
@@ -47,6 +49,75 @@ __all__ = [
 
 
 TileScopeVerificationStrategy = VerificationStrategy[Tiling, GriddedPerm]
+BasisAwareVerificationStrategyType = TypeVar(
+    "BasisAwareVerificationStrategyType", bound="BasisAwareVerificationStrategy"
+)
+
+
+class BasisAwareVerificationStrategy(TileScopeVerificationStrategy):
+    """
+    A base class for a verification strategy that needs to know the basis the
+    Tilescope is currently running.
+    """
+
+    def __init__(
+        self,
+        basis: Optional[Iterable[Perm]] = None,
+        symmetry: bool = False,
+        ignore_parent: bool = True,
+    ):
+        self._basis = tuple(basis) if basis is not None else tuple()
+        self._symmetry = symmetry
+        assert all(
+            isinstance(p, Perm) for p in self._basis
+        ), "Element of the basis must be Perm"
+        if symmetry:
+            self.symmetries = frozenset(
+                frozenset(b) for b in all_symmetry_sets(self._basis)
+            )
+        else:
+            self.symmetries = frozenset([frozenset(self._basis)])
+        super().__init__(ignore_parent=ignore_parent)
+
+    def change_basis(
+        self: BasisAwareVerificationStrategyType, basis: Iterable[Perm], symmetry: bool
+    ) -> BasisAwareVerificationStrategyType:
+        """
+        Return a new version of the verification strategy with the given basis instead
+        of the current one.
+        """
+        basis = tuple(basis)
+        return self.__class__(basis, symmetry, self.ignore_parent)
+
+    @property
+    def basis(self) -> Tuple[Perm, ...]:
+        return self._basis
+
+    def to_jsonable(self) -> dict:
+        d: dict = super().to_jsonable()
+        d["basis"] = self._basis
+        d["symmetry"] = self._symmetry
+        return d
+
+    @classmethod
+    def from_dict(
+        cls: Type[BasisAwareVerificationStrategyType], d: dict
+    ) -> BasisAwareVerificationStrategyType:
+        if "basis" in d and d["basis"] is not None:
+            basis: Optional[List[Perm]] = [Perm(p) for p in d.pop("basis")]
+        else:
+            basis = d.pop("basis", None)
+        return cls(basis=basis, **d)
+
+    def __repr__(self) -> str:
+        args = ", ".join(
+            [
+                f"basis={self._basis}",
+                f"symmetry={self._symmetry}",
+                f"ignore_parent={self.ignore_parent}",
+            ]
+        )
+        return f"{self.__class__.__name__}({args})"
 
 
 class BasicVerificationStrategy(AtomStrategy):
@@ -117,50 +188,22 @@ class BasicVerificationStrategy(AtomStrategy):
             ] = assumption.get_value(gp)
         return reduce(mul, [var(k) ** n for k, n in expected.items()], 1)
 
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}()"
 
-class OneByOneVerificationStrategy(TileScopeVerificationStrategy):
-    def __init__(
-        self,
-        basis: Optional[Iterable[Perm]] = None,
-        symmetry: bool = False,
-        ignore_parent: bool = True,
-    ):
-        self._basis = tuple(basis) if basis is not None else tuple()
-        self._symmetry = symmetry
-        assert all(
-            isinstance(p, Perm) for p in self._basis
-        ), "Element of the basis must be Perm"
-        if symmetry:
-            self.symmetries = set(frozenset(b) for b in all_symmetry_sets(self._basis))
-        else:
-            self.symmetries = set([frozenset(self._basis)])
-        super().__init__(ignore_parent=ignore_parent)
-
-    def change_basis(
-        self, basis: Iterable[Perm], symmetry: bool
-    ) -> "OneByOneVerificationStrategy":
-        """
-        Return a new version of the verfication strategy with the given basis instead of
-        the current one.
-        """
-        basis = tuple(basis)
-        return self.__class__(basis, symmetry, self.ignore_parent)
-
-    @property
-    def basis(self) -> Tuple[Perm, ...]:
-        return self._basis
 
+class OneByOneVerificationStrategy(BasisAwareVerificationStrategy):
     @staticmethod
-    def pack(tiling: Tiling) -> StrategyPack:
-        if any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions):
+    def pack(comb_class: Tiling) -> StrategyPack:
+        if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
             )
         # pylint: disable=import-outside-toplevel
         from tilings.tilescope import TileScopePack
 
-        assert tiling.dimensions == (1, 1)
-        basis, _ = tiling.cell_basis()[(0, 0)]
+        assert comb_class.dimensions == (1, 1)
+        basis, _ = comb_class.cell_basis()[(0, 0)]
         if any(
             any(p.contains(patt) for patt in basis)
             for p in [
@@ -179,49 +222,51 @@ class OneByOneVerificationStrategy(TileScopeVerificationStrategy):
         if is_insertion_encodable_rightmost(basis):
             return TileScopePack.regular_insertion_encoding(2)
         # if it is the class or positive class
-        if not tiling.requirements or (
-            len(tiling.requirements) == 1
-            and len(tiling.requirements[0]) == 1
-            and len(tiling.requirements[0][0]) <= 2
+        if not comb_class.requirements or (
+            len(comb_class.requirements) == 1
+            and len(comb_class.requirements[0]) == 1
+            and len(comb_class.requirements[0][0]) <= 2
         ):
             if basis in ([Perm((0, 1, 2))], [Perm((2, 1, 0))]):
                 # Av(123) or Av(321) - use fusion!
                 return (
                     TileScopePack.row_and_col_placements(row_only=True)
                     .make_fusion(tracked=True)
-                    .fix_one_by_one(basis)
+                    .add_basis(basis)
                 )
             if (Perm((0, 1, 2)) in basis or Perm((2, 1, 0)) in basis) and all(
                 len(p) <= 4 for p in basis
             ):
                 # is a subclass of Av(123) avoiding patterns of length <= 4
                 # experimentally showed that such clsses always terminates
-                return TileScopePack.row_and_col_placements().fix_one_by_one(basis)
+                return TileScopePack.row_and_col_placements().add_basis(basis)
         raise InvalidOperationError(
             "Cannot get a specification for one by one verification for "
             f"subclass Av({basis})"
         )
 
-    def verified(self, tiling: Tiling) -> bool:
-        return tiling.dimensions == (1, 1) and (
-            frozenset(ob.patt for ob in tiling.obstructions) not in self.symmetries
-            or any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions)
+    def verified(self, comb_class: Tiling) -> bool:
+        return comb_class.dimensions == (1, 1) and (
+            frozenset(ob.patt for ob in comb_class.obstructions) not in self.symmetries
+            or any(
+                isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions
+            )
         )
 
     def get_genf(
-        self, tiling: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
+        self, comb_class: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
     ) -> Expr:
-        if not self.verified(tiling):
+        if not self.verified(comb_class):
             raise StrategyDoesNotApply("tiling not 1x1 verified")
-        if len(tiling.obstructions) == 1 and tiling.obstructions[0] in (
+        if len(comb_class.obstructions) == 1 and comb_class.obstructions[0] in (
             GriddedPerm.single_cell((0, 1, 2), (0, 0)),
             GriddedPerm.single_cell((2, 1, 0), (0, 0)),
         ):
-            return LocalEnumeration(tiling).get_genf(funcs=funcs)
+            return LocalEnumeration(comb_class).get_genf(funcs=funcs)
         try:
-            return super().get_genf(tiling, funcs)
+            return super().get_genf(comb_class, funcs)
         except InvalidOperationError:
-            return LocalEnumeration(tiling).get_genf(funcs=funcs)
+            return LocalEnumeration(comb_class).get_genf(funcs=funcs)
 
     @staticmethod
     def formal_step() -> str:
@@ -248,30 +293,9 @@ class OneByOneVerificationStrategy(TileScopeVerificationStrategy):
             "Not implemented random sample for one by one verified tilings"
         )
 
-    def __repr__(self) -> str:
-        if self.symmetries:
-            return self.__class__.__name__ + (
-                "(basis={}, symmetry={}, " "ignore_parent={})"
-            ).format(list(self._basis), True, self.ignore_parent)
-        return self.__class__.__name__ + "()"
-
     def __str__(self) -> str:
         return "one by one verification"
 
-    def to_jsonable(self) -> dict:
-        d: dict = super().to_jsonable()
-        d["basis"] = self._basis
-        d["symmetry"] = self._symmetry
-        return d
-
-    @classmethod
-    def from_dict(cls, d: dict) -> "OneByOneVerificationStrategy":
-        if d["basis"] is not None:
-            basis: Optional[List[Perm]] = [Perm(p) for p in d.pop("basis")]
-        else:
-            basis = d.pop("basis")
-        return cls(basis=basis, **d)
-
 
 class DatabaseVerificationStrategy(TileScopeVerificationStrategy):
     """
@@ -282,26 +306,26 @@ class DatabaseVerificationStrategy(TileScopeVerificationStrategy):
     """
 
     @staticmethod
-    def pack(tiling: Tiling) -> StrategyPack:
+    def pack(comb_class: Tiling) -> StrategyPack:
         # TODO: check database for tiling
         raise InvalidOperationError(
             "Cannot get a specification for a tiling in the database"
         )
 
     @staticmethod
-    def verified(tiling: Tiling):
-        return DatabaseEnumeration(tiling).verified()
+    def verified(comb_class: Tiling):
+        return DatabaseEnumeration(comb_class).verified()
 
     @staticmethod
     def formal_step() -> str:
         return "tiling is in the database"
 
     def get_genf(
-        self, tiling: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
+        self, comb_class: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
     ) -> Expr:
-        if not self.verified(tiling):
+        if not self.verified(comb_class):
             raise StrategyDoesNotApply("tiling is not in the database")
-        return DatabaseEnumeration(tiling).get_genf()
+        return DatabaseEnumeration(comb_class).get_genf()
 
     @staticmethod
     def get_terms(comb_class: Tiling, n: int) -> Terms:
@@ -331,7 +355,7 @@ class DatabaseVerificationStrategy(TileScopeVerificationStrategy):
         return cls(**d)
 
 
-class LocallyFactorableVerificationStrategy(TileScopeVerificationStrategy):
+class LocallyFactorableVerificationStrategy(BasisAwareVerificationStrategy):
     """
     Verification strategy for a locally factorable tiling.
 
@@ -345,8 +369,8 @@ class LocallyFactorableVerificationStrategy(TileScopeVerificationStrategy):
     """
 
     @staticmethod
-    def pack(tiling: Tiling) -> StrategyPack:
-        if any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions):
+    def pack(comb_class: Tiling) -> StrategyPack:
+        if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
             )
@@ -379,21 +403,49 @@ class LocallyFactorableVerificationStrategy(TileScopeVerificationStrategy):
         reqs = chain.from_iterable(tiling.requirements)
         return all(not r.is_interleaving() for r in reqs)
 
-    def verified(self, tiling: Tiling):
+    def verified(self, comb_class: Tiling):
         return (
-            not tiling.dimensions == (1, 1)
-            and self._locally_factorable_obstructions(tiling)
-            and self._locally_factorable_requirements(tiling)
+            not comb_class.dimensions == (1, 1)
+            and self._locally_factorable_obstructions(comb_class)
+            and self._locally_factorable_requirements(comb_class)
         )
 
+    def decomposition_function(
+        self, comb_class: Tiling
+    ) -> Optional[Tuple[Tiling, ...]]:
+        """
+        The rule as the root as children if one of the cell of the tiling is the root.
+        """
+        if self.verified(comb_class):
+            if not self.basis:
+                return ()
+            sfs = locally_factorable_shift.shift_from_spec(
+                comb_class, self, self.symmetries
+            )
+            if sfs is not None:
+                return (Tiling.from_perms(self.basis),)
+            return ()
+        return None
+
+    def shifts(
+        self, comb_class: Tiling, children: Optional[Tuple[Tiling, ...]] = None
+    ) -> Tuple[int, ...]:
+        if children is None:
+            children = self.decomposition_function(comb_class)
+            if children is None:
+                raise StrategyDoesNotApply
+        if not children:
+            return ()
+        shift = locally_factorable_shift.shift_from_spec(
+            comb_class, self, self.symmetries
+        )
+        assert shift is not None
+        return (shift,)
+
     @staticmethod
     def formal_step() -> str:
         return "tiling is locally factorable"
 
-    @classmethod
-    def from_dict(cls, d: dict) -> "LocallyFactorableVerificationStrategy":
-        return cls(**d)
-
     def __str__(self) -> str:
         return "locally factorable verification"
 
@@ -413,8 +465,8 @@ class ElementaryVerificationStrategy(LocallyFactorableVerificationStrategy):
     """
 
     @staticmethod
-    def verified(tiling: Tiling):
-        return tiling.fully_isolated() and not tiling.dimensions == (1, 1)
+    def verified(comb_class: Tiling):
+        return comb_class.fully_isolated() and not comb_class.dimensions == (1, 1)
 
     @staticmethod
     def formal_step() -> str:
@@ -440,16 +492,16 @@ class LocalVerificationStrategy(TileScopeVerificationStrategy):
         self.no_factors = no_factors
         super().__init__(ignore_parent=ignore_parent)
 
-    def pack(self, tiling: Tiling) -> StrategyPack:
+    def pack(self, comb_class: Tiling) -> StrategyPack:
         try:
-            return InsertionEncodingVerificationStrategy().pack(tiling)
+            return InsertionEncodingVerificationStrategy().pack(comb_class)
         except StrategyDoesNotApply:
             pass
         if self.no_factors:
             raise InvalidOperationError("Cannot get a simpler specification")
         if (
-            any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions)
-            and len(tiling.find_factors()) == 1
+            any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions)
+            and len(comb_class.find_factors()) == 1
         ):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
@@ -468,11 +520,11 @@ class LocalVerificationStrategy(TileScopeVerificationStrategy):
             name="factor pack",
         )
 
-    def verified(self, tiling: Tiling) -> bool:
+    def verified(self, comb_class: Tiling) -> bool:
         return (
-            tiling.dimensions != (1, 1)
-            and (not self.no_factors or len(tiling.find_factors()) == 1)
-            and LocalEnumeration(tiling).verified()
+            comb_class.dimensions != (1, 1)
+            and (not self.no_factors or len(comb_class.find_factors()) == 1)
+            and LocalEnumeration(comb_class).verified()
         )
 
     @staticmethod
@@ -484,19 +536,19 @@ class LocalVerificationStrategy(TileScopeVerificationStrategy):
         return cls(**d)
 
     def get_genf(
-        self, tiling: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
+        self, comb_class: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
     ) -> Expr:
-        if not self.verified(tiling):
+        if not self.verified(comb_class):
             raise StrategyDoesNotApply("tiling not locally verified")
-        if len(tiling.obstructions) == 1 and tiling.obstructions[0] in (
+        if len(comb_class.obstructions) == 1 and comb_class.obstructions[0] in (
             GriddedPerm.single_cell((0, 1, 2), (0, 0)),
             GriddedPerm.single_cell((2, 1, 0), (0, 0)),
         ):
-            return LocalEnumeration(tiling).get_genf(funcs=funcs)
+            return LocalEnumeration(comb_class).get_genf(funcs=funcs)
         try:
-            return super().get_genf(tiling, funcs)
+            return super().get_genf(comb_class, funcs)
         except InvalidOperationError:
-            return LocalEnumeration(tiling).get_genf(funcs=funcs)
+            return LocalEnumeration(comb_class).get_genf(funcs=funcs)
 
     @staticmethod
     def get_terms(comb_class: Tiling, n: int) -> Terms:
@@ -530,17 +582,17 @@ class InsertionEncodingVerificationStrategy(TileScopeVerificationStrategy):
     def __init__(self, ignore_parent: bool = True):
         super().__init__(ignore_parent=ignore_parent)
 
-    def pack(self, tiling: Tiling) -> StrategyPack:
-        if any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions):
+    def pack(self, comb_class: Tiling) -> StrategyPack:
+        if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
             )
         # pylint: disable=import-outside-toplevel
         from tilings.strategy_pack import TileScopePack
 
-        if self.has_rightmost_insertion_encoding(tiling):
+        if self.has_rightmost_insertion_encoding(comb_class):
             return TileScopePack.regular_insertion_encoding(2)
-        if self.has_topmost_insertion_encoding(tiling):
+        if self.has_topmost_insertion_encoding(comb_class):
             return TileScopePack.regular_insertion_encoding(3)
         raise StrategyDoesNotApply("tiling does not has a regular insertion encoding")
 
@@ -558,10 +610,10 @@ class InsertionEncodingVerificationStrategy(TileScopeVerificationStrategy):
             for basis, _ in tiling.cell_basis().values()
         )
 
-    def verified(self, tiling: Tiling) -> bool:
+    def verified(self, comb_class: Tiling) -> bool:
         return self.has_rightmost_insertion_encoding(
-            tiling
-        ) or self.has_topmost_insertion_encoding(tiling)
+            comb_class
+        ) or self.has_topmost_insertion_encoding(comb_class)
 
     @staticmethod
     def formal_step() -> str:
@@ -606,19 +658,17 @@ class MonotoneTreeVerificationStrategy(TileScopeVerificationStrategy):
         self.no_factors = no_factors
         super().__init__(ignore_parent=ignore_parent)
 
-    def pack(self, tiling: Tiling) -> StrategyPack:
-        if any(isinstance(ass, ComponentAssumption) for ass in tiling.assumptions):
+    def pack(self, comb_class: Tiling) -> StrategyPack:
+        if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
             )
         try:
-            return InsertionEncodingVerificationStrategy().pack(tiling)
+            return InsertionEncodingVerificationStrategy().pack(comb_class)
         except StrategyDoesNotApply:
             pass
         if self.no_factors:
-            raise InvalidOperationError(
-                "Cannot get a specification for a tiling in the database"
-            )
+            raise InvalidOperationError("Cannot get a simpler specification")
         return StrategyPack(
             initial_strats=[FactorFactory()],
             inferral_strats=[],
@@ -632,10 +682,10 @@ class MonotoneTreeVerificationStrategy(TileScopeVerificationStrategy):
             name="factor pack",
         )
 
-    def verified(self, tiling: Tiling) -> bool:
+    def verified(self, comb_class: Tiling) -> bool:
         return (
-            not self.no_factors or len(tiling.find_factors()) == 1
-        ) and MonotoneTreeEnumeration(tiling).verified()
+            not self.no_factors or len(comb_class.find_factors()) == 1
+        ) and MonotoneTreeEnumeration(comb_class).verified()
 
     @staticmethod
     def formal_step() -> str:
@@ -646,14 +696,14 @@ class MonotoneTreeVerificationStrategy(TileScopeVerificationStrategy):
         return cls(**d)
 
     def get_genf(
-        self, tiling: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
+        self, comb_class: Tiling, funcs: Optional[Dict[Tiling, Function]] = None
     ) -> Expr:
-        if not self.verified(tiling):
+        if not self.verified(comb_class):
             raise StrategyDoesNotApply("tiling not locally verified")
         try:
-            return super().get_genf(tiling, funcs)
+            return super().get_genf(comb_class, funcs)
         except InvalidOperationError:
-            return MonotoneTreeEnumeration(tiling).get_genf(funcs=funcs)
+            return MonotoneTreeEnumeration(comb_class).get_genf(funcs=funcs)
 
     @staticmethod
     def get_terms(comb_class: Tiling, n: int) -> Terms:
